      program tbpac_SA

      use printing

      implicit none

      character*80 line,infile,outfile
      character*60 title
      integer method,grads,dunit,eunit,charge,natom
      integer i,spc,spc2,iargc
      logical ldone,lgeom,debug
      parameter (debug=.true.)
      external iargc

      i=iargc()

      if(i.ge.1)call getarg(1,infile)
      if(i.ge.2)then
         call getarg(2,outfile)
      else
         spc=index(infile,' ')
	 outfile=infile(1:spc-1)//'.out'
      endif

      funit=2
      open(unit=1,file=infile)
      open(unit=funit,file=outfile)

      ldone=.false.
      lgeom=.false.

c Let''s set some defaults
      title=' '
      method=11
      grads=0
      printlev=2
      dunit=1
      eunit=1
      charge=0
      natom=0
      cutCi=0.8d0
      cutMO=0.6d0
      cutEj=1.0d-8

      do while (.not.ldone)
        read(1,'(A80)',end=9999)line
	call lc(line)
	select case(line(1:5))
	 case('title')
	   spc=index(line,' ')
	   title=line(7:66)
	 case('metho')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)method
	 case('grads')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)grads
	 case('print')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)printlev
	   if(printlev.gt.4)then
	      printlev=3
	      write(funit,*)'Cautionary Errror: Value for PRINT too high!'
	      write(funit,*)'    Using PRINT level 3 (high) instead.'
	   endif
	 case('unitd')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)dunit
	 case('unite')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)eunit
	 case('charg')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)charge
	 case('natom')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)natom
	 case('cutmo')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)cutMO
	 case('cutci')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)cutCi
	 case('cutei')
	   spc=index(line,' ')
	   spc2=spc+index(line(spc+1:80),' ')
	   read(line(spc:spc2),*)cutEj
	 case('geome')
	    if(natom.eq.0)Stop'Define natom before geometry directive'
	    call getgeom(natom,method,grads,dunit,eunit,charge,title)
	    ldone=.true.
	 case default
	    write(funit,*)'The following line could not be understood ',
     *           'and was thus ignored.'
            write(funit,*)line
	end select 
 12   enddo
      close(funit)
      goto 1
 9999 continue
      write(funit,*)'End of file reached before GEOMETRY ',
     *   'directive found.'
      write(funit,*)'Stopping.'
 1    continue
      end program tbpac_SA

      subroutine getgeom(natom,method,grads,dunit,eunit,charge,title)

      use params
      use printing

      implicit none
      
      character*80 line,infile,outfile
      character*60 title
      integer method,grads,dunit,eunit,charge,natom
      integer i,ndim
      logical ldone,lgeom
      double precision x(natom),y(natom),z(natom)
      double precision dx(natom),dy(natom),dz(natom)
      double precision xx,yy,zz,VV
      double precision qn(natom)
      character*2 symbol(natom)
      character*8 printstring
      double precision Ecomp(3)

c Determine the conversions between dunit and atomic units
      select case(dunit)
      case(1)
         unitd=1.889727d0
	 unitdsym='Ang'
c         write(2,*)'debug1, unitd=',unitd,'dunit=',dunit
      case(2)
         unitd=1.0d0
	 unitdsym='a.u.'
c         write(2,*)'debug2, unitd=',unitd,'dunit=',dunit
      case default
         Stop 'Undefined value for directive UNITD'
      end select

c Determine the conversions between eunit and atomic units
      select case(eunit)
      case(1)
         unite=0.03674901d0
	 unitesym='eV      '
      case(2)
         unite=0.001593601d0
	 unitesym='kcal/mol '
      case(3)
         unite=1.0d0
	 unitesym='kJ/mol  '
	 Stop 'UNITE=3 not yet defined.'
      case(4)
         unite=1.0d0
	 unitesym='cm-1    '
	 Stop 'UNITE=4 not yet defined.'
      case(5)
         unite=1.0d0
	 unitesym='Ha      '
      case default
         Stop 'Undefined value for directive UNITE'
      end select

      if(printlev.ge.1)then
        call printheader(2)
        write(funit,10)'Title: ',title
	write(funit,11)'Method',method
	if(printlev.ge.2)call printmethod(method)
	if(grads.eq.0)then
	   write(funit,*)'Gradients: NO'
	else
	   write(funit,*)'Gradients: YES'
	endif
	select case(printlev)
	case(1)
	   printstring='minimal'
	case(2)
	   printstring='standard'
	case(3)
	   printstring='high'
	case(4)
	   printstring='debug'
	end select
	write(funit,10)'Print Level:',printstring
	write(funit,10)'Energy Units:',unitesym
	write(funit,10)'Distance Units:',unitdsym
	write(funit,12)'MO Print Cutoff:',cutMO
	write(funit,12)'CSF Print Cutoff:',cutCi
	write(funit,11)'Total Charge:',charge
	write(funit,11)'Number of Atoms:',natom
      endif

      if(printlev.ge.2)write(funit,*)
      if(printlev.ge.2)write(funit,*)'Input Geometry in Units of ',
     *       unitdsym
      do i=1,natom
         read(1,*,end=9999)symbol(i),xx,yy,zz
	 if(printlev.ge.2)write(funit,13)i,symbol(i),xx,yy,zz
	 x(i)=xx*unitd
	 y(i)=yy*unitd
	 z(i)=zz*unitd
      enddo
      if(printlev.ge.2)write(funit,*)

      close(1)
      if((printlev.ge.4).and.(dunit.ne.2))then
         write(funit,*)'Input Geometry in Atomic Units:'
	 do i=1,natom
	    write(funit,13)i,symbol(i),x(i),y(i),z(i)
	 enddo
	 write(funit,*)
      endif
      write(funit,*)

      call tbpac(natom,natom,x,y,z,symbol,VV,dx,dy,dz,grads,method,
     *   charge)

      goto 9990
      
 9999 Stop 'END OF FILE reached before NATOM lines read.'

 9990 continue

 10   format(A,' ',A)
 11   format(A,' ',I4)
 12   format(A,' ',F6.4)
 13   format(I5,2X,A,3(2X,F12.6))
 100  format(A,F10.5,A)
 101  format(2X,I4,2X,F8.4)

      end subroutine getgeom

      subroutine tbpac(natom,siz,x,y,z,symbol,v,dx,dy,dz,grads,method,
     *   charge)
      use basics
      use grad
      use params
      use printing
      implicit none
      integer natom,ndim,nElect,i,idtype(natom),grads,siz,method,charge
      integer stoich(maxid)
      double precision x(siz),y(siz),z(siz),dx(siz),dy(siz),dz(siz)
      double precision v,xx(natom),yy(natom),zz(natom)
      double precision dEdx(natom),dEdy(natom),dEdz(natom)
      double precision Tzero,vsip(maxid,2)
      character*2 symbol(siz),symb(natom)
      integer met2icase
      external met2icase
      common /vsips/ vsip

      if(siz.lt.natom)Stop 'size is less than natom'

      icase=met2icase(method)

      select case(icase)
      case (1:8)
         vsip(1,1)=parvsip(icase,1,1)
         vsip(3,1)=parvsip(icase,3,1)
         vsip(3,2)=parvsip(icase,3,2)
      case (9:24)
         vsip(1,1)=vsip_exp(1,1)
         vsip(3,1)=vsip_exp(3,1)
         vsip(3,2)=vsip_exp(3,2)
      case (25)
         vsip=vsip_exp
      case default
         write(*,*)'icase = ',icase
         stop 'Hmmm - vsip?'
      end select

      ndim=0
      nElect=-charge
      Tzero=0.0d0
      stoich=0
      do i=1,natom
         xx(i)=x(i)
	 yy(i)=y(i)
	 zz(i)=z(i)
	 symb(i)=symbol(i)
	 select case (symb(i))
	 case('AL','Al','aL','al','13')
	    idtype(i)=3
	    stoich(3)=stoich(3)+1
	    ndim=ndim+norbit(idtype(i))
	    nElect=nElect+nelem(idtype(i))
	    if(icase.le.8)then
               Tzero=Tzero+parTzero(icase)
	    else
	       Tzero=Tzero+2.0d0*vsip(3,1)+vsip(3,2)+parUpen(icase)
	    endif
         case(' H','H ',' h','h ',' 1','1 ')
	    idtype(i)=1
	    stoich(1)=stoich(1)+1
	    ndim=ndim+norbit(idtype(i))
	    nElect=nElect+nelem(idtype(i))
	       Tzero=Tzero+vsip(3,1)
         case(' C','C ',' c','c ',' 6','6 ')
	    idtype(i)=2
	    stoich(2)=stoich(2)+1
	    ndim=ndim+norbit(2)
	    nElect=nElect+nelem(2)
	       Tzero=Tzero+2.0d0*(vsip(2,1)+vsip(2,2))+parUpen(icase)
	 case('NI','Ni','nI','ni','28')
	    idtype(i)=4
	    stoich(4)=stoich(4)+1
	    ndim=ndim+norbit(4)
	    nElect=nElect+nelem(4)
	    Tzero=Tzero+10.0d0*vsip(4,2)+5.0d0*parUpen(icase)
	 case('CU','Cu','cU','cu','29')
	    idtype(i)=5
	    stoich(5)=stoich(5)+1
	    ndim=ndim+norbit(5)
	    nElect=nElect+nelem(5)
	    Tzero=Tzero+10.0d0*vsip(5,2)+vsip(5,1)+5.0d0*parUpen(icase)
	 case('PD','Pd','pD','pd','46')
	    idtype(i)=6
	    stoich(6)=stoich(6)+1
	    ndim=ndim+norbit(6)
	    nElect=nElect+nelem(6)
	    Tzero=Tzero+10.0d0*vsip(6,2)+5.0d0*parUpen(icase)
	 case('AG','Ag','aG','ag','47')
	    idtype(i)=7
	    stoich(7)=stoich(7)+1
	    ndim=ndim+norbit(7)
	    nElect=nElect+nelem(7)
	    Tzero=Tzero+10.0d0*vsip(7,2)+vsip(7,1)+5.0d0*parUpen(icase)
	 case('PT','Pt','pT','pt','78')
	    idtype(i)=8
	    stoich(8)=stoich(8)+1
	    ndim=ndim+norbit(8)
	    nElect=nElect+nelem(8)
	    Tzero=Tzero+10.0d0*vsip(8,2)+5.0d0*parUpen(icase)
	 case('AU','Au','aU','au','79')
	    idtype(i)=9
	    stoich(9)=stoich(9)+1
	    ndim=ndim+norbit(9)
	    nElect=nElect+nelem(9)
	    Tzero=Tzero+10.0d0*vsip(9,2)+vsip(9,1)+5.0d0*parUpen(icase)
         case default
	    write(*,*)'Atomic symbol ',symb(i),
     *        ' not recognized/parameterised.'
	    Stop 'Unrecognized atomic symbol. Cannot continue. Sorry.'
	 end select
      enddo

      call checkvalid(method,stoich)

      if(grads.eq.0)then
        dograd=.false.
	ndimdH=1
	ndimat=1
      else
        dograd=.true.
	ndimdH=ndim
	ndimat=natom
      endif
      flag=.false.

      select case(icase)
      case(18:20,23:24)
         ndimS=ndim
	 doS=.true.
	 if(dograd)then
	   ndimdS1=ndim
	   ndimdS2=natom
	 else
	   ndimdS1=1
	   ndimdS2=1
	 endif
      case default
         ndimS=1
	 doS=.false.
	 ndimdS1=1
	 ndimdS2=1
      end select

      call surf(natom,ndim,nElect,Tzero,xx,yy,zz,idtype,
     &   dEdx,dEdy,dEdz,v)

      if(dograd)then
      do i=1,natom
         dx(i)=dEdx(i)
         dy(i)=dEdy(i)
         dz(i)=dEdz(i)
      enddo
      endif
      end subroutine tbpac
      
      subroutine lc (string)
c
c   Subroutine which takes a string of 5 characters and converts the 
c   upper case letters in the string to lower case letters
cMAI: Borrowed from Polyrate 9.4.2
c
      implicit none
      character*80 string
      character*1 xlett
      integer i,itry

      do 10 i = 1,5
         xlett=string(i:i)
         itry=ichar(xlett)
         if(xlett.ge.'A'.and.xlett.le.'Z') then
              itry=itry+32
              string(i:i)=char(itry)
         endif
 10   continue

      end subroutine lc

      subroutine printheader(f)

      implicit none
      integer f

      write(f,*)'******************************************************'
      write(f,*)'*               TBPAC - version 3.0                  *'
      write(f,*)'*                                                    *'
      write(f,*)'*                September 5, 2007                   *'
      write(f,*)'*                                                    *'
      write(f,*)'*       By: Mark A. Iron, Grazyna Staszewska,        *'
      write(f,*)'*           Tiqing Liu, Ahren W. Jasper,             *'
      write(f,*)'*               and Donald G. Truhlar                *'
      write(f,*)'*                                                    *'
      write(f,*)'*               University of Minnesota,             *'
      write(f,*)'*                Minneapolis, MN 55455.              *'
      write(f,*)'*                                                    *'
      write(f,*)'******************************************************'
      write(f,*)
      write(f,*)'******************************************************'
      write(f,*)'*   Use of this program only per terms of the user   *'
      write(f,*)'*    agreement. Please see the User''s Manual.   ',
     &  '     *'
      write(f,*)'*                                                    *'
      write(f,*)'* For more details, please visit the TBPAC homepage: *'
      write(f,*)'*                                                    *'
      write(f,*)'*          http://comp.chem.umn.edu/tbpac/           *'
      write(f,*)'*                                                    *'
      write(f,*)'******************************************************'
      write(f,*)
      write(f,*)

      end subroutine printheader
      
      subroutine printmethod(met)
      use printing 
      implicit none
      integer met,ic

      select case(met)
      case(11)
        write(funit,1)'TBCI-S, DELTA=0.1 eV'
      case(21)
        ic=5
      case(31)
        ic=8
      case(41)
        ic=9
      case(42)
        ic=10
      case(43)
        ic=11
      case(44)
        ic=12
      case(45)
        ic=13
      case(46)
        ic=25
      case(51)
        ic=14
      case(52)
        ic=15
      case(53)
        ic=16
      case(54)
        ic=17
      case(55)
        ic=21
      case(56)
        ic=22
      case(61)
        ic=23
      case(62)
        ic=24
      case(63)
        ic=18
      case(64)
        ic=19
      case(65)
        ic=10
      case default
         write(*,*)'METHOD = ',met
         Stop 'Undefined value for directive METHOD'
      end select

 1    format(' Description: ',A)

      end subroutine printmethod

