      subroutine tbpac(natom,siz,x,y,z,symbol,v,dx,dy,dz,grads,method,
     *   charge)
      use basics
      use grad
      use params
      use printing
      implicit none
      integer natom,ndim,nElect,i,idtype(natom),grads,siz,method,charge
      integer stoich(maxid)
      double precision x(siz),y(siz),z(siz),dx(siz),dy(siz),dz(siz)
      double precision v,xx(natom),yy(natom),zz(natom)
      double precision dEdx(natom),dEdy(natom),dEdz(natom)
      double precision Tzero,vsip(maxid,2)
      character*2 symbol(siz),symb(natom)
      integer met2icase
      external met2icase
      common /vsips/ vsip

c***************************************
c Need to define some system parameters
c***************************************

c Uncomment one of the following lines as needed
c If want to work in eV
c      unite=0.03674901d0
c If want to work in a.u. (i.e., Hartrees)
c      unite=1.0d0

c Uncomment one of the following lines as needed
c If want to work in Angstoms
c      unitd=1.889727d0
c If want to work in a.u. (i.e., Bohr)
c      unitd=1.0d0

c define level of output (0=none)
      printlev=0
c define which file unit to print to
c   if not opened, then open it
      funit=2
c default TBCI parameter: cut-off in the CI treatment
      cutEj=1d-8

      if(siz.lt.natom)Stop 'size is less than natom'

      icase=met2icase(method)

      select case(icase)
      case (1:8)
         vsip(1,1)=parvsip(icase,1,1)
         vsip(3,1)=parvsip(icase,3,1)
         vsip(3,2)=parvsip(icase,3,2)
      case (9:24)
         vsip(1,1)=vsip_exp(1,1)
         vsip(3,1)=vsip_exp(3,1)
         vsip(3,2)=vsip_exp(3,2)
      case (25)
         vsip=vsip_exp
      case default
         write(*,*)'icase = ',icase
         stop 'Hmmm - vsip?'
      end select

      ndim=0
      nElect=-charge
      Tzero=0.0d0
      stoich=0
      do i=1,natom
c Convert units
         xx(i)=x(i)*unitd
	 yy(i)=y(i)*unitd
	 zz(i)=z(i)*unitd
	 symb(i)=symbol(i)
	 select case (symb(i))
	 case('AL','Al','aL','al','13')
	    idtype(i)=3
	    stoich(3)=stoich(3)+1
	    ndim=ndim+norbit(idtype(i))
	    nElect=nElect+nelem(idtype(i))
	    if(icase.le.8)then
               Tzero=Tzero+parTzero(icase)
	    else
	       Tzero=Tzero+2.0d0*vsip(3,1)+vsip(3,2)+parUpen(icase)
	    endif
         case(' H','H ',' h','h ',' 1','1 ')
	    idtype(i)=1
	    stoich(1)=stoich(1)+1
	    ndim=ndim+norbit(idtype(i))
	    nElect=nElect+nelem(idtype(i))
	       Tzero=Tzero+vsip(3,1)
         case(' C','C ',' c','c ',' 6','6 ')
	    idtype(i)=2
	    stoich(2)=stoich(2)+1
	    ndim=ndim+norbit(2)
	    nElect=nElect+nelem(2)
	       Tzero=Tzero+2.0d0*(vsip(2,1)+vsip(2,2))+parUpen(icase)
	 case('NI','Ni','nI','ni','28')
	    idtype(i)=4
	    stoich(4)=stoich(4)+1
	    ndim=ndim+norbit(4)
	    nElect=nElect+nelem(4)
	    Tzero=Tzero+10.0d0*vsip(4,2)+5.0d0*parUpen(icase)
	 case('CU','Cu','cU','cu','29')
	    idtype(i)=5
	    stoich(5)=stoich(5)+1
	    ndim=ndim+norbit(5)
	    nElect=nElect+nelem(5)
	    Tzero=Tzero+10.0d0*vsip(5,2)+vsip(5,1)+5.0d0*parUpen(icase)
	 case('PD','Pd','pD','pd','46')
	    idtype(i)=6
	    stoich(6)=stoich(6)+1
	    ndim=ndim+norbit(6)
	    nElect=nElect+nelem(6)
	    Tzero=Tzero+10.0d0*vsip(6,2)+5.0d0*parUpen(icase)
	 case('AG','Ag','aG','ag','47')
	    idtype(i)=7
	    stoich(7)=stoich(7)+1
	    ndim=ndim+norbit(7)
	    nElect=nElect+nelem(7)
	    Tzero=Tzero+10.0d0*vsip(7,2)+vsip(7,1)+5.0d0*parUpen(icase)
	 case('PT','Pt','pT','pt','78')
	    idtype(i)=8
	    stoich(8)=stoich(8)+1
	    ndim=ndim+norbit(8)
	    nElect=nElect+nelem(8)
	    Tzero=Tzero+10.0d0*vsip(8,2)+5.0d0*parUpen(icase)
	 case('AU','Au','aU','au','79')
	    idtype(i)=9
	    stoich(9)=stoich(9)+1
	    ndim=ndim+norbit(9)
	    nElect=nElect+nelem(9)
	    Tzero=Tzero+10.0d0*vsip(9,2)+vsip(9,1)+5.0d0*parUpen(icase)
         case default
	    write(*,*)'Atomic symbol ',symb(i),
     *        ' not recognized/parameterised.'
	    Stop 'Unrecognized atomic symbol. Cannot continue. Sorry.'
	 end select
      enddo

      call checkvalid(method,stoich)

      if(grads.eq.0)then
        dograd=.false.
	ndimdH=1
	ndimat=1
      else
        dograd=.true.
	ndimdH=ndim
	ndimat=natom
      endif
      flag=.false.

      select case(icase)
      case(18:20,23:24)
         ndimS=ndim
	 doS=.true.
	 if(dograd)then
	   ndimdS1=ndim
	   ndimdS2=natom
	 else
	   ndimdS1=1
	   ndimdS2=1
	 endif
      case default
         ndimS=1
	 doS=.false.
	 ndimdS1=1
	 ndimdS2=1
      end select

      call surf(natom,ndim,nElect,Tzero,xx,yy,zz,idtype,
     &   dEdx,dEdy,dEdz,v)

      v=v/unite
      if(dograd)then
      do i=1,natom
         dx(i)=dEdx(i)*unitd/unite
         dy(i)=dEdy(i)*unitd/unite
         dz(i)=dEdz(i)*unitd/unite
      enddo
      endif
      end subroutine tbpac
